// Type definitions for obs-parser 0.5.0
// Project: https://github.com/jincdream/obs-parser
// Definitions by: jinc.cjc <jinc.cjc@alibaba-inc.com> <https://github.com/jincdream>
// Definitions: https://github.com/jincdream/obs-parser
// TypeScript Version: ^3.7.0

export declare type Base = {
    [key: string]: object;
};
export declare type ComponentDetail<T extends Base> = {
    [P in keyof T]?: T[P];
};
export declare type ComponentData<T> = {
    [P in keyof T]?: FiledData<T[P], EffectFields<T>, EffectFields<T>>;
};
export declare type EffectHandle = {
    /**
     * onChnage
     */
    trigger?: string;
    /**
     * $state.type === 'content'
     * $state：当前组件的state
     * $value：trigger的回调值
     */
    condition?: string;
    handle?: {
        /**
         * uid
         */
        targetUid: string;
        type: 'get' | 'set' | 'visible';
        value?: {
            /**
             * key: "$value"
             */
            [key: string]: string;
        };
    };
};
export declare type Effect = {
    /**
     * componentId
     */
    uid?: string;
    effects?: EffectHandle[];
};
export declare type FiledData<T, E, S> = {
    effect?: {
        enble: boolean;
        handles?: EffectHandle[];
    };
    effectFields?: E;
    scriptFields?: S;
    fields?: T;
};
export interface IData {
    id?: string;
    status?: number;
    resource?: object;
    type?: string;
    [key: string]: any;
}
export declare type EffectFields<C> = {
    [P in keyof C[keyof C]]: string;
};
export interface IComponentRenderDO<AllComponents, ComponentsData extends {
    style?: object;
}> {
    id: keyof AllComponents;
    n: keyof ComponentsData;
    e?: Effect;
    l?: Linkages<keyof ComponentsData[keyof ComponentsData]>;
    s?: EffectFields<ComponentsData>;
    d: Partial<ComponentsData>;
    childrens: (IComponentRenderDO<AllComponents, ComponentsData>)[] | [];
}
export declare type Structure<T> = {
    [P in keyof T]: Array<keyof T>;
};
export declare type Linkages<U> = Array<{
    exp: string;
    deps: Array<U>;
    target: U;
}>;
export interface OBS_Schema<Components extends Base, AllComponents extends Base> {
    data: ComponentData<AllComponents>;
    endpoint?: {
        mode?: string;
        pageCode?: string;
        protocolVersion?: string;
    };
    hierarchy: {
        component?: Array<keyof AllComponents>;
        componentDetail?: ComponentDetail<Components>;
        root: keyof AllComponents;
        structure: Structure<AllComponents>;
    };
}
export default function OBS_DataParser<ComponentsData extends Base, AllComponents extends Base>(schema: OBS_Schema<ComponentsData, AllComponents>): IComponentRenderDO<AllComponents, ComponentsData>[];
//# sourceMappingURL=index.d.ts.map